"use strict";

var _chai = require("chai");

var _codemirror = _interopRequireDefault(require("codemirror"));

require("codemirror/addon/hint/show-hint");

var _graphql = require("graphql");

require("../hint");

var _testSchema = require("./testSchema");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function createEditorWithHint() {
  return (0, _codemirror["default"])(document.createElement('div'), {
    mode: 'graphql',
    hintOptions: {
      schema: _testSchema.TestSchema,
      closeOnUnfocus: false,
      completeSingle: false
    }
  });
}

function getHintSuggestions(queryString, cursor) {
  var editor = createEditorWithHint();
  return new Promise(function (resolve) {
    var graphqlHint = _codemirror["default"].hint.graphql;

    _codemirror["default"].hint.graphql = function (cm, options) {
      var result = graphqlHint(cm, options);
      resolve(result);
      _codemirror["default"].hint.graphql = graphqlHint;
      return result;
    };

    editor.doc.setValue(queryString);
    editor.doc.setCursor(cursor);
    editor.execCommand('autocomplete');
  });
}

function checkSuggestions(source, suggestions) {
  var titles = suggestions.map(function (suggestion) {
    return suggestion.text;
  });
  (0, _chai.expect)(titles).to.deep.equal(source);
}

describe('graphql-hint', function () {
  it('attaches a GraphQL hint function with correct mode/hint options',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee() {
    var editor;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return createEditorWithHint();

          case 2:
            editor = _context.sent;
            (0, _chai.expect)(editor.getHelpers(editor.getCursor(), 'hint')).to.not.have.lengthOf(0);

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  })));
  it('provides correct initial keywords',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2() {
    var suggestions, initialKeywords;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return getHintSuggestions('', {
              line: 0,
              ch: 0
            });

          case 2:
            suggestions = _context2.sent;
            initialKeywords = ['query', 'mutation', 'subscription', 'fragment', '{'];
            checkSuggestions(initialKeywords, suggestions.list);

          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  })));
  it('provides correct field name suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3() {
    var suggestions, fieldConfig, fieldNames, fieldTypes, expectedTypes;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return getHintSuggestions('{ ', {
              line: 0,
              ch: 2
            });

          case 2:
            suggestions = _context3.sent;
            fieldConfig = _testSchema.TestSchema.getQueryType().getFields();
            fieldNames = Object.keys(fieldConfig).filter(function (name) {
              return !fieldConfig[name].isDeprecated;
            });
            checkSuggestions(fieldNames.concat(['__typename', '__schema', '__type']), suggestions.list);
            fieldTypes = fieldNames.map(function (name) {
              return fieldConfig[name].type;
            });
            expectedTypes = suggestions.list.filter(function (item) {
              return item.text !== '__schema' && item.text !== '__type' && item.text !== '__typename';
            }).map(function (item) {
              return item.type;
            });
            (0, _chai.expect)(fieldTypes).to.deep.equal(expectedTypes);

          case 9:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  })));
  it('provides correct field name suggestions when using aliases',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee4() {
    var suggestions, fieldConfig;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return getHintSuggestions('{ aliasTest: first { ', {
              line: 0,
              ch: 21
            });

          case 2:
            suggestions = _context4.sent;
            fieldConfig = _testSchema.TestSchema.getType('First').getFields();
            checkSuggestions([].concat(_toConsumableArray(Object.keys(fieldConfig)), ['__typename']), suggestions.list);

          case 5:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  })));
  it('provides correct field name suggestion indentation',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee5() {
    var suggestions;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return getHintSuggestions('{\n  ', {
              line: 1,
              ch: 2
            });

          case 2:
            suggestions = _context5.sent;
            (0, _chai.expect)(suggestions.from).to.deep.equal({
              line: 1,
              ch: 2,
              sticky: null
            });
            (0, _chai.expect)(suggestions.to).to.deep.equal({
              line: 1,
              ch: 2,
              sticky: null
            });

          case 5:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  })));
  it('provides correct argument suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee6() {
    var suggestions, argumentNames;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return getHintSuggestions('{ hasArgs ( ', {
              line: 0,
              ch: 12
            });

          case 2:
            suggestions = _context6.sent;
            argumentNames = _testSchema.TestSchema.getQueryType().getFields().hasArgs.args.map(function (arg) {
              return arg.name;
            });
            checkSuggestions(argumentNames, suggestions.list);

          case 5:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  })));
  it('provides correct argument suggestions when using aliases',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee7() {
    var suggestions, argumentNames;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return getHintSuggestions('{ aliasTest: hasArgs ( ', {
              line: 0,
              ch: 23
            });

          case 2:
            suggestions = _context7.sent;
            argumentNames = _testSchema.TestSchema.getQueryType().getFields().hasArgs.args.map(function (arg) {
              return arg.name;
            });
            checkSuggestions(argumentNames, suggestions.list);

          case 5:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  })));
  it('provides correct directive suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee8() {
    var suggestions, directiveNames;
    return regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return getHintSuggestions('{ test (@', {
              line: 0,
              ch: 9
            });

          case 2:
            suggestions = _context8.sent;
            directiveNames = ['include', 'skip'];
            checkSuggestions(directiveNames, suggestions.list);

          case 5:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  })));
  it('provides correct directive suggestions when using aliases',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee9() {
    var suggestions, directiveNames;
    return regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return getHintSuggestions('{ aliasTest: test (@', {
              line: 0,
              ch: 20
            });

          case 2:
            suggestions = _context9.sent;
            directiveNames = ['include', 'skip'];
            checkSuggestions(directiveNames, suggestions.list);

          case 5:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9);
  })));
  it('provides correct directive suggestions on definitions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee10() {
    var suggestions, directiveNames;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return getHintSuggestions('type Type @', {
              line: 0,
              ch: 11
            });

          case 2:
            suggestions = _context10.sent;
            directiveNames = ['onAllDefs'];
            checkSuggestions(directiveNames, suggestions.list);

          case 5:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  })));
  it('provides correct directive suggestions on args definitions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee11() {
    var suggestions, directiveNames;
    return regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return getHintSuggestions('type Type { field(arg: String @', {
              line: 0,
              ch: 31
            });

          case 2:
            suggestions = _context11.sent;
            directiveNames = ['onArg', 'onAllDefs'];
            checkSuggestions(directiveNames, suggestions.list);

          case 5:
          case "end":
            return _context11.stop();
        }
      }
    }, _callee11);
  })));
  it('provides correct typeCondition suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee12() {
    var suggestions, unionType, typeConditionNames;
    return regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return getHintSuggestions('{ union { ... on ', {
              line: 0,
              ch: 17
            });

          case 2:
            suggestions = _context12.sent;
            unionType = _testSchema.TestSchema.getQueryType().getFields().union.type;
            typeConditionNames = _testSchema.TestSchema.getPossibleTypes(unionType).map(function (type) {
              return type.name;
            }).concat(['TestInterface']);
            checkSuggestions(typeConditionNames, suggestions.list);

          case 6:
          case "end":
            return _context12.stop();
        }
      }
    }, _callee12);
  })));
  it('provides correct typeCondition suggestions on fragment',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee13() {
    var suggestions, typeMap, typeConditionNames;
    return regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return getHintSuggestions('fragment Foo on ', {
              line: 0,
              ch: 16
            });

          case 2:
            suggestions = _context13.sent;
            typeMap = _testSchema.TestSchema.getTypeMap();
            typeConditionNames = Object.keys(typeMap).filter(function (typeName) {
              var type = typeMap[typeName];
              return (0, _graphql.isCompositeType)(type);
            });
            checkSuggestions(typeConditionNames, suggestions.list);

          case 6:
          case "end":
            return _context13.stop();
        }
      }
    }, _callee13);
  })));
  it('provides correct ENUM suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee14() {
    var suggestions, enumNames;
    return regeneratorRuntime.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            _context14.next = 2;
            return getHintSuggestions('{ hasArgs (enum: ', {
              line: 0,
              ch: 17
            });

          case 2:
            suggestions = _context14.sent;
            enumNames = _testSchema.TestSchema.getType('TestEnum').getValues().map(function (value) {
              return value.name;
            });
            checkSuggestions(enumNames, suggestions.list);

          case 5:
          case "end":
            return _context14.stop();
        }
      }
    }, _callee14);
  })));
  it('provides correct testInput suggestions',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee15() {
    var suggestions, testInputNames;
    return regeneratorRuntime.wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            _context15.next = 2;
            return getHintSuggestions('{ hasArgs (object: { ', {
              line: 0,
              ch: 21
            });

          case 2:
            suggestions = _context15.sent;
            testInputNames = Object.keys(_testSchema.TestSchema.getType('TestInput').getFields());
            checkSuggestions(testInputNames, suggestions.list);

          case 5:
          case "end":
            return _context15.stop();
        }
      }
    }, _callee15);
  })));
  it('provides fragment name suggestion',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee16() {
    var suggestions;
    return regeneratorRuntime.wrap(function _callee16$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            _context16.next = 2;
            return getHintSuggestions('fragment Foo on Test { id }  query { ...', {
              line: 0,
              ch: 40
            });

          case 2:
            suggestions = _context16.sent;
            checkSuggestions(['Foo'], suggestions.list);

          case 4:
          case "end":
            return _context16.stop();
        }
      }
    }, _callee16);
  })));
  it('provides fragment names for fragments defined lower',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee17() {
    var suggestions;
    return regeneratorRuntime.wrap(function _callee17$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            _context17.next = 2;
            return getHintSuggestions('query { ... } fragment Foo on Test { id }', {
              line: 0,
              ch: 11
            });

          case 2:
            suggestions = _context17.sent;
            checkSuggestions(['Foo'], suggestions.list);

          case 4:
          case "end":
            return _context17.stop();
        }
      }
    }, _callee17);
  })));
  it('provides only appropriate fragment names',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee18() {
    var suggestions;
    return regeneratorRuntime.wrap(function _callee18$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            _context18.next = 2;
            return getHintSuggestions('fragment Foo on TestUnion { ... } ' + 'fragment Bar on First { name } ' + 'fragment Baz on Second { name } ' + 'fragment Qux on TestUnion { name } ' + 'fragment Nrf on Test { id }', {
              line: 0,
              ch: 31
            });

          case 2:
            suggestions = _context18.sent;
            checkSuggestions(['Bar', 'Baz', 'Qux'], suggestions.list);

          case 4:
          case "end":
            return _context18.stop();
        }
      }
    }, _callee18);
  })));
  it('provides correct field name suggestion inside inline fragment',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee19() {
    var suggestions, fieldNames;
    return regeneratorRuntime.wrap(function _callee19$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            _context19.next = 2;
            return getHintSuggestions('fragment Foo on TestUnion { ... on First { ', {
              line: 0,
              ch: 43
            });

          case 2:
            suggestions = _context19.sent;
            fieldNames = Object.keys(_testSchema.TestSchema.getType('First').getFields());
            fieldNames.push('__typename');
            checkSuggestions(fieldNames, suggestions.list);

          case 6:
          case "end":
            return _context19.stop();
        }
      }
    }, _callee19);
  })));
  it('provides correct field name suggestion inside typeless inline fragment',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee20() {
    var suggestions, fieldNames;
    return regeneratorRuntime.wrap(function _callee20$(_context20) {
      while (1) {
        switch (_context20.prev = _context20.next) {
          case 0:
            _context20.next = 2;
            return getHintSuggestions('fragment Foo on First { ... { ', {
              line: 0,
              ch: 30
            });

          case 2:
            suggestions = _context20.sent;
            fieldNames = Object.keys(_testSchema.TestSchema.getType('First').getFields());
            fieldNames.push('__typename');
            checkSuggestions(fieldNames, suggestions.list);

          case 6:
          case "end":
            return _context20.stop();
        }
      }
    }, _callee20);
  })));
});