"use strict";

var _chai = require("chai");

var _codemirror = _interopRequireDefault(require("codemirror"));

require("codemirror/addon/lint/lint");

require("../lint");

var _testSchema = require("./testSchema");

var _fs = require("fs");

var _path = require("path");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function createEditorWithLint(lintConfig) {
  return (0, _codemirror["default"])(document.createElement('div'), {
    mode: 'graphql',
    lint: lintConfig ? lintConfig : true
  });
}

function printLintErrors(queryString) {
  var editor = createEditorWithLint({
    schema: _testSchema.TestSchema
  });
  return new Promise(function (resolve) {
    editor.state.lint.options.onUpdateLinting = function (errors) {
      if (errors && errors[0]) {
        if (!errors[0].message.match('Unexpected EOF')) {
          resolve(errors);
        }
      }

      resolve([]);
    };

    editor.doc.setValue(queryString);
  });
}

describe('graphql-lint', function () {
  it('attaches a GraphQL lint function with correct mode/lint options', function () {
    var editor = createEditorWithLint();
    (0, _chai.expect)(editor.getHelpers(editor.getCursor(), 'lint')).to.not.have.lengthOf(0);
  });
  var kitchenSink = (0, _fs.readFileSync)((0, _path.join)(__dirname, '/kitchen-sink.graphql'), {
    encoding: 'utf8'
  });
  it('returns no syntactic/validation errors after parsing kitchen-sink query',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee() {
    var errors;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return printLintErrors(kitchenSink);

          case 2:
            errors = _context.sent;
            (0, _chai.expect)(errors).to.have.lengthOf(0);

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  })));
});