"use strict";

var _chai = require("chai");

var _codemirror = _interopRequireDefault(require("codemirror"));

require("codemirror/addon/runmode/runmode");

require("../mode");

var _fs = require("fs");

var _path = require("path");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 *  Copyright (c) 2019 GraphQL Contributors
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */
describe('graphql-mode', function () {
  it('provides correct tokens and styles after parsing', function () {
    var queryStr = 'query name { }';
    var tokens = [];
    var styles = [];

    _codemirror["default"].runMode(queryStr, 'graphql', function (token, style) {
      if (style && style !== 'ws') {
        tokens.push(token);
        styles.push(style);
      }
    });

    (0, _chai.expect)(tokens).to.deep.equal(['query', 'name', '{', '}']);
    (0, _chai.expect)(styles).to.deep.equal(['keyword', 'def', 'punctuation', 'punctuation']);
  });
  it('parses Relay-style anonymous FragmentDefinitions', function () {
    _codemirror["default"].runMode('fragment on Test { id }', 'graphql', function (token, style) {
      return (0, _chai.expect)(style).to.not.equal('invalidchar');
    });
  });
  it('parses inline fragments with optional syntax correctly', function () {
    _codemirror["default"].runMode('{ ... on OptionalType { name } }', 'graphql', function (token, style) {
      return (0, _chai.expect)(style).to.not.equal('invalidchar');
    });

    _codemirror["default"].runMode('{ ... { name } }', 'graphql', function (token, style) {
      return (0, _chai.expect)(style).to.not.equal('invalidchar');
    });

    _codemirror["default"].runMode('{ ... @optionalDirective { name } }', 'graphql', function (token, style) {
      return (0, _chai.expect)(style).to.not.equal('invalidchar');
    });
  });
  it('returns "invalidchar" message when there is no matching token', function () {
    _codemirror["default"].runMode('qauery name', 'graphql', function (token, style) {
      if (token.trim()) {
        (0, _chai.expect)(style).to.equal('invalidchar');
      }
    });

    _codemirror["default"].runMode('query %', 'graphql', function (token, style) {
      if (token === '%') {
        (0, _chai.expect)(style).to.equal('invalidchar');
      }
    });
  });
  it('parses kitchen-sink query without invalidchar', function () {
    var kitchenSink = (0, _fs.readFileSync)((0, _path.join)(__dirname, '/kitchen-sink.graphql'), {
      encoding: 'utf8'
    });

    _codemirror["default"].runMode(kitchenSink, 'graphql', function (token, style) {
      (0, _chai.expect)(style).to.not.equal('invalidchar');
    });
  });
  it('parses schema-kitchen-sink query without invalidchar', function () {
    var schemaKitchenSink = (0, _fs.readFileSync)((0, _path.join)(__dirname, '/schema-kitchen-sink.graphql'), {
      encoding: 'utf8'
    });

    _codemirror["default"].runMode(schemaKitchenSink, 'graphql', function (token, style) {
      (0, _chai.expect)(style).to.not.equal('invalidchar');
    });
  });
  it('parses anonymous operations without invalidchar', function () {
    _codemirror["default"].runMode('{ id }', 'graphql', function (token, style) {
      (0, _chai.expect)(style).to.not.equal('invalidchar');
    });

    _codemirror["default"].runMode("\n      mutation {\n        setString(value: \"newString\")\n      }\n    ", 'graphql', function (token, style) {
      (0, _chai.expect)(style).to.not.equal('invalidchar');
    });

    _codemirror["default"].runMode("\n      subscription {\n        subscribeToTest(id: \"anId\") {\n          id\n        }\n      }\n    ", 'graphql', function (token, style) {
      (0, _chai.expect)(style).to.not.equal('invalidchar');
    });
  });
});