"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestSchema = void 0;

var _graphql = require("graphql");

/**
 *  Copyright (c) 2019 GraphQL Contributors
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */
// Test Schema
var TestEnum = new _graphql.GraphQLEnumType({
  name: 'TestEnum',
  values: {
    RED: {},
    GREEN: {},
    BLUE: {}
  }
});
var TestInputObject = new _graphql.GraphQLInputObjectType({
  name: 'TestInput',
  fields: function fields() {
    return {
      string: {
        type: _graphql.GraphQLString
      },
      "int": {
        type: _graphql.GraphQLInt
      },
      "float": {
        type: _graphql.GraphQLFloat
      },
      "boolean": {
        type: _graphql.GraphQLBoolean
      },
      id: {
        type: _graphql.GraphQLID
      },
      "enum": {
        type: TestEnum
      },
      object: {
        type: TestInputObject
      },
      // List
      listString: {
        type: new _graphql.GraphQLList(_graphql.GraphQLString)
      },
      listInt: {
        type: new _graphql.GraphQLList(_graphql.GraphQLInt)
      },
      listFloat: {
        type: new _graphql.GraphQLList(_graphql.GraphQLFloat)
      },
      listBoolean: {
        type: new _graphql.GraphQLList(_graphql.GraphQLBoolean)
      },
      listID: {
        type: new _graphql.GraphQLList(_graphql.GraphQLID)
      },
      listEnum: {
        type: new _graphql.GraphQLList(TestEnum)
      },
      listObject: {
        type: new _graphql.GraphQLList(TestInputObject)
      }
    };
  }
});
var TestInterface = new _graphql.GraphQLInterfaceType({
  name: 'TestInterface',
  resolveType: function resolveType() {
    return UnionFirst;
  },
  fields: {
    scalar: {
      type: _graphql.GraphQLString,
      resolve: function resolve() {
        return {};
      }
    }
  }
});
var UnionFirst = new _graphql.GraphQLObjectType({
  name: 'First',
  interfaces: [TestInterface],
  fields: function fields() {
    return {
      scalar: {
        type: _graphql.GraphQLString,
        resolve: function resolve() {
          return {};
        }
      },
      first: {
        type: TestType,
        resolve: function resolve() {
          return {};
        }
      }
    };
  }
});
var UnionSecond = new _graphql.GraphQLObjectType({
  name: 'Second',
  fields: function fields() {
    return {
      second: {
        type: TestType,
        resolve: function resolve() {
          return {};
        }
      }
    };
  }
});
var TestUnion = new _graphql.GraphQLUnionType({
  name: 'TestUnion',
  types: [UnionFirst, UnionSecond],
  resolveType: function resolveType() {
    return UnionFirst;
  }
});
var TestType = new _graphql.GraphQLObjectType({
  name: 'Test',
  fields: function fields() {
    return {
      test: {
        type: TestType,
        resolve: function resolve() {
          return {};
        }
      },
      deprecatedTest: {
        type: TestType,
        deprecationReason: 'Use test instead.',
        resolve: function resolve() {
          return {};
        }
      },
      union: {
        type: TestUnion,
        resolve: function resolve() {
          return {};
        }
      },
      first: {
        type: UnionFirst,
        resolve: function resolve() {
          return {};
        }
      },
      id: {
        type: _graphql.GraphQLInt,
        resolve: function resolve() {
          return {};
        }
      },
      isTest: {
        type: _graphql.GraphQLBoolean,
        resolve: function resolve() {
          return true;
        }
      },
      hasArgs: {
        type: _graphql.GraphQLString,
        resolve: function resolve(value, args) {
          return JSON.stringify(args);
        },
        args: {
          string: {
            type: _graphql.GraphQLString
          },
          "int": {
            type: _graphql.GraphQLInt
          },
          "float": {
            type: _graphql.GraphQLFloat
          },
          "boolean": {
            type: _graphql.GraphQLBoolean
          },
          id: {
            type: _graphql.GraphQLID
          },
          "enum": {
            type: TestEnum
          },
          object: {
            type: TestInputObject
          },
          // List
          listString: {
            type: new _graphql.GraphQLList(_graphql.GraphQLString)
          },
          listInt: {
            type: new _graphql.GraphQLList(_graphql.GraphQLInt)
          },
          listFloat: {
            type: new _graphql.GraphQLList(_graphql.GraphQLFloat)
          },
          listBoolean: {
            type: new _graphql.GraphQLList(_graphql.GraphQLBoolean)
          },
          listID: {
            type: new _graphql.GraphQLList(_graphql.GraphQLID)
          },
          listEnum: {
            type: new _graphql.GraphQLList(TestEnum)
          },
          listObject: {
            type: new _graphql.GraphQLList(TestInputObject)
          }
        }
      }
    };
  }
});
var TestMutationType = new _graphql.GraphQLObjectType({
  name: 'MutationType',
  description: 'This is a simple mutation type',
  fields: {
    setString: {
      type: _graphql.GraphQLString,
      description: 'Set the string field',
      args: {
        value: {
          type: _graphql.GraphQLString
        }
      }
    }
  }
});
var TestSubscriptionType = new _graphql.GraphQLObjectType({
  name: 'SubscriptionType',
  description: 'This is a simple subscription type',
  fields: {
    subscribeToTest: {
      type: TestType,
      description: 'Subscribe to the test type',
      args: {
        id: {
          type: _graphql.GraphQLString
        }
      }
    }
  }
});
var OnArgDirective = new _graphql.GraphQLDirective({
  name: 'onArg',
  locations: [_graphql.DirectiveLocation.ARGUMENT_DEFINITION]
});
var OnAllDefsDirective = new _graphql.GraphQLDirective({
  name: 'onAllDefs',
  locations: [_graphql.DirectiveLocation.SCHEMA, _graphql.DirectiveLocation.SCALAR, _graphql.DirectiveLocation.OBJECT, _graphql.DirectiveLocation.FIELD_DEFINITION, _graphql.DirectiveLocation.INTERFACE, _graphql.DirectiveLocation.UNION, _graphql.DirectiveLocation.ENUM, _graphql.DirectiveLocation.ENUM_VALUE, _graphql.DirectiveLocation.INPUT_OBJECT, _graphql.DirectiveLocation.ARGUMENT_DEFINITION, _graphql.DirectiveLocation.INPUT_FIELD_DEFINITION]
});
var TestSchema = new _graphql.GraphQLSchema({
  query: TestType,
  mutation: TestMutationType,
  subscription: TestSubscriptionType,
  directives: [_graphql.GraphQLIncludeDirective, _graphql.GraphQLSkipDirective, _graphql.GraphQLDeprecatedDirective, OnArgDirective, OnAllDefsDirective]
});
exports.TestSchema = TestSchema;